//From bildr article: http://bildr.org/2012/08/rotary-encoder-arduino/
//Adapted for use with an Arduino MEGA in order to use two encoders,
//  which needed 4 interrupt lines.


int buttonPin = 28;
int buttonState = 0;

//Piezo Pin
int piezoPin = A0;
bool piezoHit = false;
int piezoInputThreshold = 5; //an analog read will give plain values
//the input will need to be above the threshold to register as a hit
//adjust as needed to reduce false positives

//Piezo LED
int piezoLED = 22;


//Encoder 1
//these pins can not be changed 2/3 are interrupt pins
int encoder1Pin1 = 2;
int encoder1Pin2 = 3;

volatile int lastEncoded1 = 0;
volatile long encoder1Value = 0;

long lastEncoder1Value = 0;

int lastMSB1 = 0;
int lastLSB1 = 0;

bool turningRight1 = false;
bool turningLeft1 = false;

//Encoder 1 LED
int encoder1LED= 24;



//Encoder 2
//these pins can not be changed 18/19 are interrupt pins
int encoder2Pin1 = 18;
int encoder2Pin2 = 19;

volatile int lastEncoded2 = 0;
volatile long encoder2Value = 0;

long lastEncoder2Value = 0;

int lastMSB2 = 0;
int lastLSB2 = 0;

bool turningRight2 = false;
bool turningLeft2 = false;

//Encoder 2 LED
int encoder2LED = 26;



int reduceTurnSensitivity = 6;

int tonePin = 8;

int score[] = {

  //Starting value of 'no note' (zero index)
  1,
  //I have a pen (4 notes)
  184,138,138,138,
  //I have an apple (5 notes)
  207,138,138,138,138,
  //*bang bang* (2 notes)
  207,138,
  //Apple  (2 notes)
  207,138,
  //Pen! (1 note)
  138,
  //(2nd) I have a pen (4 notes)
  184,138,138,138,
  //I have a pineapple (5 notes)
  207,138,138,138,138,
  //*bang bang* (2 notes)
  207,138,
  //Pineapple (3 notes) (4? the original had an extra 138, double check this sounds right)
  207,138,207,
  //Pen! (1 note)
  138,
  //Apple (2 notes)
  164,164,
  //Pen (1 note)
  164,
  //Pineapple (3 notes)
  207,207,207,
  //Pen (1 note)
  207,
  //*bang bang* (2 notes)
  207,138,
  //Pen (1 note)
  207,
  //Pineapple (3 notes)
  138,138,130,
  //Apple (2 notes)
  138,146,
  //Pen (1 notes)
  138

};


//doesn't seem to be an array.length type function for arduino
//hard coding the amount of notes in this array
//sizeof() was intersting... but not giving the proper amount of notes
//might be worth revisiting
int scoreLength = 46;
int scorePosition = 0; //the individual note we are on right now
int songPart = 0;      //the line of the song we are on
int noteDuration = 135;


void setup() {
  Serial.begin (9600);
  Serial.print("Length of the score:");
  Serial.println(scoreLength);


  
  

  //Set up first encoder
  //
  pinMode(encoder1Pin1, INPUT); 
  pinMode(encoder1Pin2, INPUT);

  digitalWrite(encoder1Pin1, HIGH); //turn pullup resistor on
  digitalWrite(encoder1Pin2, HIGH); //turn pullup resistor on

  //call updateEncoder() when any high/low changed seen
  //on interrupt 0 (pin 2), or interrupt 1 (pin 3) 
  attachInterrupt(0, updateEncoder1, CHANGE); 
  attachInterrupt(1, updateEncoder1, CHANGE);

  pinMode(encoder1LED,OUTPUT);



  //Set up second encoder
  //
  pinMode(encoder2Pin1, INPUT); 
  pinMode(encoder2Pin2, INPUT);

  digitalWrite(encoder2Pin1, HIGH); //turn pullup resistor on
  digitalWrite(encoder2Pin2, HIGH); //turn pullup resistor on

  //call updateEncoder() when any high/low changed seen
  //on interrupt 5 (pin 18), or interrupt 4 (pin 19) 
  attachInterrupt(5, updateEncoder2, CHANGE); 
  attachInterrupt(4, updateEncoder2, CHANGE);

  pinMode(encoder2LED,OUTPUT);


  //no setup needed for piezo input, but needs the LED
  pinMode(piezoLED,OUTPUT);

}

void loop(){ 

    
  
  Serial.print("Turning Left:");
  Serial.print(turningLeft1);
  Serial.print(" Turning Right:");
  Serial.print(turningRight1);
  Serial.print(" Score position:");
  Serial.print(scorePosition);
  Serial.print(" Song part:");
  Serial.print(songPart);
  Serial.print(" encoderValue:");
  Serial.print(encoder2Value);
  //Serial.print(analogRead(piezoPin));

  buttonState = digitalRead(buttonPin);

  if (buttonState == LOW){
    songPart = 0;
    scorePosition = 0;
    Serial.println("Button Pressed");
  }else{
    //nothing
  }
  
  Serial.print(" Lyric: ");
  if (scorePosition == 0){
    Serial.print("::starting game::");
    
    songPart = 0;
    digitalWrite(encoder1LED, LOW);
    digitalWrite(encoder2LED, LOW);
    digitalWrite(piezoLED, HIGH);

    //ledDance();
  }
  else if (scorePosition >= 1 && scorePosition <= 4){
    Serial.print("I have a pen");
    songPart = 1;
    digitalWrite(encoder1LED, LOW);
    digitalWrite(encoder2LED, LOW);
    digitalWrite(piezoLED, HIGH);
  }
  else if (scorePosition >= 5 && scorePosition <= 9){
    Serial.print("I have an apple");
    songPart = 2;
    digitalWrite(encoder1LED, HIGH);
    digitalWrite(encoder2LED, LOW);
    digitalWrite(piezoLED, LOW);
  }
  else if (scorePosition >= 10 && scorePosition <= 11){
    Serial.print("*bang bang*");
    digitalWrite(encoder1LED, LOW);
    digitalWrite(encoder2LED, LOW);
    digitalWrite(piezoLED, HIGH);
    songPart = 3;
  }
  else if (scorePosition >= 12 && scorePosition <= 13){
    Serial.print("Apple");
    digitalWrite(encoder1LED, HIGH);
    digitalWrite(encoder2LED, LOW);
    digitalWrite(piezoLED, LOW);
    songPart = 4;
  }
  else if (scorePosition == 14){
    Serial.print("Pen!");
    digitalWrite(encoder1LED, LOW);
    digitalWrite(encoder2LED, LOW);
    digitalWrite(piezoLED, HIGH);
    songPart = 5;
  }
  else if (scorePosition >= 15 && scorePosition <= 18){
    Serial.print("(2nd) I have a pen");
    digitalWrite(encoder1LED, LOW);
    digitalWrite(encoder2LED, LOW);
    digitalWrite(piezoLED, HIGH);
    songPart = 6;
  }
  else if (scorePosition >= 18 && scorePosition <= 24){
    Serial.print("I have a pineapple");
    digitalWrite(encoder1LED, LOW);
    digitalWrite(encoder2LED, HIGH);
    digitalWrite(piezoLED, LOW);
    songPart = 7;
  }
  else if (scorePosition >= 25 && scorePosition <= 26){
    Serial.print("*bang bang*");
    digitalWrite(encoder1LED, LOW);
    digitalWrite(encoder2LED, LOW);
    digitalWrite(piezoLED, HIGH);
    songPart = 8;
  }
  else if (scorePosition >= 27 && scorePosition <= 29){
    Serial.print("Pineapple");
    digitalWrite(encoder1LED, LOW);
    digitalWrite(encoder2LED, HIGH);
    digitalWrite(piezoLED, LOW);
    songPart = 9;
  }
  else if (scorePosition == 30){
    Serial.print("Pen!");
    digitalWrite(encoder1LED, LOW);
    digitalWrite(encoder2LED, LOW);
    digitalWrite(piezoLED, HIGH);
    songPart = 10;
  }
  else if (scorePosition >= 31 && scorePosition <= 32){
    Serial.print("Apple");
    digitalWrite(encoder1LED, HIGH);
    digitalWrite(encoder2LED, LOW);
    digitalWrite(piezoLED, LOW);
    songPart = 11;
  }
  else if (scorePosition == 33){
    Serial.print("Pen");
    digitalWrite(encoder1LED, LOW);
    digitalWrite(encoder2LED, LOW);
    digitalWrite(piezoLED, HIGH);
    songPart = 12;
  }
  else if (scorePosition >= 34 && scorePosition <= 36){
    Serial.print("Pineapple");
    digitalWrite(encoder1LED, LOW);
    digitalWrite(encoder2LED, HIGH);
    digitalWrite(piezoLED, LOW);
    songPart = 13;
  }
  else if (scorePosition == 37){
    Serial.print("Pen");
    digitalWrite(encoder1LED, LOW);
    digitalWrite(encoder2LED, LOW);
    digitalWrite(piezoLED, HIGH);
    songPart = 14;
  }
  else if (scorePosition >= 38 && scorePosition <= 39){
    Serial.print("*bang bang*");
    digitalWrite(encoder1LED, LOW);
    digitalWrite(encoder2LED, LOW);
    digitalWrite(piezoLED, HIGH);
    songPart = 15;
  }
  else if (scorePosition == 40){
    Serial.print("Pen");
    digitalWrite(encoder1LED, LOW);
    digitalWrite(encoder2LED, LOW);
    digitalWrite(piezoLED, HIGH);
    songPart = 16;
  }
  else if (scorePosition >= 41 && scorePosition <= 43){
    Serial.print("Pineapple");
    digitalWrite(encoder1LED, LOW);
    digitalWrite(encoder2LED, HIGH);
    digitalWrite(piezoLED, LOW);
    songPart = 17;
  }
  else if (scorePosition >= 44 && scorePosition <= 45){
    Serial.print("Apple");
    digitalWrite(encoder1LED, HIGH);
    digitalWrite(encoder2LED, LOW);
    digitalWrite(piezoLED, LOW);
    songPart = 18;
  }
  else if (scorePosition == 46){
    digitalWrite(encoder1LED, LOW);
    digitalWrite(encoder2LED, LOW);
    digitalWrite(piezoLED, HIGH);
    Serial.print("Pen");
    songPart = 19;
  }
  
  Serial.println();

  
  //delay(1000); //just here to slow down the output, and show it will work  even during a delay
  delay(100);


  
  

  //Moving the current position of the score
  //We want to move forward in the song depending on what input is given
  //We set up if statements that make sure we are inside of the scorePosition array
  //Then another if statement indicates what parts of the song are linked to that input
  //When we are on the proper part of the song and the proper input is triggered, then advance the song
  //  for the rotary encoders, left and right each need to be addressed, allowing for the song to be reversed.


  //detect if he piezo should be registered as a hit
  if (analogRead(piezoPin)>piezoInputThreshold){
    piezoHit = true;
  }

  
  //Rotary input 1; Apple
   if (turningLeft1 && scorePosition>0){
    if(songPart == 2 || songPart == 4 || songPart == 11 || songPart == 18){//So these numbers can be anything, not just sequential
      //Turning off left turn functionality for now
      scorePosition--;
      tone(tonePin,score[scorePosition],noteDuration);
       }
    }
   else if (turningRight1 && scorePosition<scoreLength){
    if(songPart == 2 || songPart == 4 || songPart == 11 || songPart == 18){//as long as the exact same ones are listed down here for the reverse functionality
      //digitalWrite(encoder1LED,HIGH);
      scorePosition++;
      tone(tonePin,score[scorePosition],noteDuration);
     }
    }
  

   //Rotary input 2; Pineapple
   if (turningLeft2 && scorePosition>0){
    if(songPart == 7 || songPart == 9 || songPart == 13 || songPart == 17){
      //Turning off left turn functionality for now
      scorePosition--;
      tone(tonePin,score[scorePosition],noteDuration);
       }
    }
   else if (turningRight2 && scorePosition<scoreLength){
    if(songPart == 7 || songPart == 9 || songPart == 13 || songPart == 17){
      //digitalWrite(encoder2LED,HIGH);
      scorePosition++;
      tone(tonePin,score[scorePosition],noteDuration);
     }
    }
         
  

  //Piezo input; Bang and Pen
  
  if (piezoHit == true && scorePosition<scoreLength){
    if( songPart ==0
    ||  songPart ==  1 || songPart == 3 || songPart == 5 || songPart == 6
    || songPart == 8 || songPart == 10 || songPart == 12 || songPart == 14
    || songPart == 15 || songPart == 16){
      //digitalWrite(piezoLED,HIGH);
      scorePosition++;
      tone(tonePin,score[scorePosition],noteDuration);
    }
    else if(songPart == 19){
      tone(tonePin,score[scorePosition],noteDuration);
      //the end!
      
    }
  }
  
  if(songPart == 19){

    Serial.println("The END!");
      
      victorySong();
      songPart = 0;
      scorePosition = 0;
    //songPart = 0;
    //victorySong();
  }



  //Reset both turn states to false for each rotary encoder at the end of the main loop
  //This makes the reporting of "turning" only temporary
  //When the knob stops moving, both states report false
  //Remove this code in order to hold the left or right values
  turningLeft1=false;
  turningRight1=false;
  //digitalWrite(encoder1LED,LOW);

  turningLeft2=false;
  turningRight2=false;
  //digitalWrite(encoder2LED,LOW);

  
  piezoHit = false;
  //digitalWrite(piezoLED,LOW);  
}


void updateEncoder1(){
  int MSB = digitalRead(encoder1Pin1); //MSB = most significant bit
  int LSB = digitalRead(encoder1Pin2); //LSB = least significant bit

  int encoded = (MSB << 1) |LSB; //converting the 2 pin value to single number
  int sum  = (lastEncoded1 << 2) | encoded; //adding it to the previous encoded value

  if(sum == 0b1101 || sum == 0b0100 || sum == 0b0010 || sum == 0b1011){
    encoder1Value ++;    
  }
  if(sum == 0b1110 || sum == 0b0111 || sum == 0b0001 || sum == 0b1000){
    //Turning off left turning for now
    encoder1Value --;
  }
 

  lastEncoded1 = encoded; //store this value for next time


  if (encoder1Value/reduceTurnSensitivity > lastEncoder1Value/reduceTurnSensitivity){
    turningRight1=true;
    turningLeft1=false;
  }
  if (encoder1Value/reduceTurnSensitivity < lastEncoder1Value/reduceTurnSensitivity){
    //Turning off left turning for now
    turningLeft1=true;
    turningRight1=false;    
  }

  //Do the same for the adjusted value, in order to test movement
  lastEncoder1Value = encoder1Value;


  /*
  Serial.print("Encoder Value:");
  Serial.print(encoderValue);
  Serial.print("Encoder Value/4:");
  Serial.println(encoderValue/4);
  */
}


void updateEncoder2(){
  int MSB = digitalRead(encoder2Pin1); //MSB = most significant bit
  int LSB = digitalRead(encoder2Pin2); //LSB = least significant bit

  int encoded = (MSB << 1) |LSB; //converting the 2 pin value to single number
  int sum  = (lastEncoded2 << 2) | encoded; //adding it to the previous encoded value

  if(sum == 0b1101 || sum == 0b0100 || sum == 0b0010 || sum == 0b1011){
    encoder2Value ++;    
  }
  if(sum == 0b1110 || sum == 0b0111 || sum == 0b0001 || sum == 0b1000){
    //Turning off left turning for now
    encoder2Value --;
  }
 

  lastEncoded2 = encoded; //store this value for next time


  if (encoder2Value/reduceTurnSensitivity > lastEncoder2Value/reduceTurnSensitivity){
    turningRight2=true;
    turningLeft2=false;
  }
  if (encoder2Value/reduceTurnSensitivity < lastEncoder2Value/reduceTurnSensitivity){
    //Turning off left turning for now
    turningLeft2=true;
    turningRight2=false;    
  }

  //Do the same for the adjusted value, in order to test movement
  lastEncoder2Value = encoder2Value;


  /*
  Serial.print("Encoder Value:");
  Serial.print(encoderValue);
  Serial.print("Encoder Value/4:");
  Serial.println(encoderValue/4);
  */
}

void ledDance(){
  int encoder1Dancing = random(0,2);
  int encoder2Dancing = random(0,2);
  int piezoDancing = random(0,2);
  
  if (encoder1Dancing == 1){
    digitalWrite(encoder1LED, HIGH);
    //delay(136);
  }
  else  {
    digitalWrite(encoder1LED, LOW);
    //delay(136);  
  }
  
  if (encoder2Dancing == 1){
    digitalWrite(encoder2LED, HIGH);
    //delay(136);
  }
  else  {
    digitalWrite(encoder2LED, LOW); 
    //delay(136); 
  }


  if (piezoDancing == 1){
    digitalWrite(piezoLED, HIGH);
    //delay(136);
    Serial.print("piezoLED true");
  }
  else  {
    digitalWrite(piezoLED, LOW);
    //delay(136);
    Serial.print("piezoLED false");  
  }
  //delay(68);
}




void victorySong() {

    tone(tonePin, 184, 134.999865);
    ledDance();
    delay(136.3635);
    delay(136.3635);
    tone(tonePin, 138, 134.999865);
    ledDance();
    delay(136.3635);
    delay(136.3635);
    tone(tonePin, 138, 134.999865);
    ledDance();
    delay(136.3635);
    delay(136.3635);
    ledDance();
    tone(tonePin, 138, 134.999865);
    delay(136.3635);
    delay(1227.2715);
    ledDance();
    tone(tonePin, 207, 134.999865);
    delay(136.3635);
    delay(136.3635);
    ledDance();
    tone(tonePin, 138, 134.999865);
    delay(136.3635);
    delay(136.3635);
    ledDance();
    tone(tonePin, 138, 134.999865);
    delay(136.3635);
    delay(136.3635);
    ledDance();
    tone(tonePin, 138, 134.999865);
    delay(136.3635);
    delay(136.3635);
    ledDance();
    tone(tonePin, 138, 134.999865);
    delay(136.3635);
    delay(954.5445);
    ledDance();
    tone(tonePin, 207, 134.999865);
    delay(136.3635);
    delay(409.0905);
    ledDance();
    tone(tonePin, 138, 134.999865);
    delay(136.3635);
    delay(954.5445);
    ledDance();
    tone(tonePin, 207, 134.999865);
    delay(136.3635);
    delay(136.3635);
    ledDance();
    tone(tonePin, 138, 134.999865);
    delay(136.3635);
    delay(409.0905);
    ledDance();
    tone(tonePin, 138, 134.999865);
    delay(136.3635);
    delay(1772.7255);
    ledDance();
    tone(tonePin, 184, 134.999865);
    delay(136.3635);
    delay(136.3635);
    ledDance();
    tone(tonePin, 138, 134.999865);
    delay(136.3635);
    delay(136.3635);
    ledDance();
    tone(tonePin, 138, 134.999865);
    delay(136.3635);
    delay(136.3635);
    ledDance();
    tone(tonePin, 138, 134.999865);
    delay(136.3635);
    delay(1227.2715);
    ledDance();
    tone(tonePin, 207, 134.999865);
    delay(136.3635);
    delay(136.3635);
    ledDance();
    tone(tonePin, 138, 134.999865);
    delay(136.3635);
    delay(136.3635);
    ledDance();
    tone(tonePin, 138, 134.999865);
    delay(136.3635);
    delay(136.3635);
    ledDance();
    tone(tonePin, 138, 134.999865);
    delay(136.3635);
    delay(136.3635);
    ledDance();
    tone(tonePin, 138, 134.999865);
    delay(136.3635);
    delay(954.5445);
    ledDance();
    tone(tonePin, 207, 134.999865);
    delay(136.3635);
    delay(409.0905);
    ledDance();
    tone(tonePin, 138, 134.999865);
    delay(136.3635);
    delay(954.5445);
    ledDance();
    tone(tonePin, 207, 134.999865);
    delay(136.3635);
    delay(136.3635);
    ledDance();
    tone(tonePin, 138, 134.999865);
    delay(136.3635);
    delay(409.0905);
    ledDance();
    tone(tonePin, 138, 134.999865);
    delay(136.3635);
    delay(1772.7255);
    ledDance();
    tone(tonePin, 164, 134.999865);
    delay(136.3635);
    delay(136.3635);
    ledDance();
    tone(tonePin, 164, 134.999865);
    delay(136.3635);
    delay(409.0905);
    ledDance();
    tone(tonePin, 164, 134.999865);
    delay(136.3635);
    delay(1227.2715);
    ledDance();
    tone(tonePin, 207, 134.999865);
    delay(136.3635);
    delay(136.3635);
    ledDance();
    tone(tonePin, 207, 134.999865);
    delay(136.3635);
    delay(136.3635);
    ledDance();
    tone(tonePin, 207, 134.999865);
    delay(136.3635);
    delay(136.3635);
    ledDance();
    tone(tonePin, 207, 134.999865);
    delay(136.3635);
    delay(1227.2715);
    ledDance();
    tone(tonePin, 207, 134.999865);
    delay(136.3635);
    delay(136.3635);
    ledDance();
    tone(tonePin, 138, 134.999865);
    delay(136.3635);
    delay(136.3635);
    ledDance();
    tone(tonePin, 138, 134.999865);
    delay(136.3635);
    delay(136.3635);
    ledDance();
    tone(tonePin, 130, 134.999865);
    delay(136.3635);
    delay(136.3635);
    ledDance();
    tone(tonePin, 138, 134.999865);
    delay(136.3635);
    delay(136.3635);
    ledDance();
    tone(tonePin, 146, 134.999865);
    delay(136.3635);
    delay(136.3635);
    ledDance();
    tone(tonePin, 138, 134.999865);
    delay(136.3635);

    //When we reach the end of the song, reset
    songPart = 0;

}
